@@ -275,13 +275,32 @@ asn1_get_octet_der (const unsigned char *der, int der_len,
   return ASN1_SUCCESS;
 }
 
-/* Returns ASN1_SUCCESS on success or an error code on error.
- */
+
+/*- 
+ * _asn1_get_time_der:
+ * @type: %ASN1_ETYPE_GENERALIZED_TIME or %ASN1_ETYPE_UTC_TIME
+ * @der: DER data to decode containing the time
+ * @der_len: Length of DER data to decode.
+ * @ret_len: Output variable containing the length of the DER data.
+ * @str: Pre-allocated output buffer to put the textual time in.
+ * @str_size: Length of pre-allocated output buffer.
+ * @flags: Zero or %ASN1_DECODE_FLAG_STRICT_DER
+ *
+ * Performs basic checks in the DER encoded time object and returns its textual form.
+ * The textual form will be in the YYYYMMDD000000Z format for GeneralizedTime
+ * and YYMMDD000000Z for UTCTime.
+ *
+ * Returns: %ASN1_SUCCESS on success, or an error.
+ -*/
 static int
-_asn1_get_time_der (const unsigned char *der, int der_len, int *ret_len,
-		    char *str, int str_size)
+_asn1_get_time_der (unsigned type, const unsigned char *der, int der_len, int *ret_len,
+		    char *str, int str_size, unsigned flags)
 {
   int len_len, str_len;
+  unsigned i;
+  unsigned sign_count = 0;
+  unsigned dot_count = 0;
+  const unsigned char *p;
 
   if (der_len <= 0 || str == NULL)
     return ASN1_DER_ERROR;
@@ -290,6 +309,50 @@ _asn1_get_time_der (const unsigned char *der, int der_len, int *ret_len,
   if (str_len <= 0 || str_size < str_len)
     return ASN1_DER_ERROR;
 
+  /* perform some sanity checks on the data */
+  if (str_len < 8)
+    {
+      warn();
+      return ASN1_DER_ERROR;
+    }
+
+  if (flags & ASN1_DECODE_FLAG_STRICT_DER)
+    {
+      p = &der[len_len];
+      for (i=0;i<(unsigned)(str_len-1);i++)
+         {
+           if (isdigit(p[i]) == 0)
+             {
+               if (type == ASN1_ETYPE_GENERALIZED_TIME)
+                 {
+                   /* tolerate lax encodings */
+                   if (p[i] == '.' && dot_count == 0)
+                     {
+                       dot_count++;
+                       continue;
+                     }
+
+               /* This is not really valid DER, but there are
+                * structures using that */
+                   if (!(flags & ASN1_DECODE_FLAG_STRICT_DER) &&
+                       (p[i] == '+' || p[i] == '-') && sign_count == 0)
+                     {
+                       sign_count++;
+                       continue;
+                     }
+                 }
+
+               warn();
+               return ASN1_DER_ERROR;
+             }
+         }
+
+      if (sign_count == 0 && p[str_len-1] != 'Z')
+        {
+          warn();
+          return ASN1_DER_ERROR;
+        }
+    }
   memcpy (str, der + len_len, str_len);
   str[str_len] = 0;
   *ret_len = str_len + len_len;
@@ -297,8 +360,20 @@ _asn1_get_time_der (const unsigned char *der, int der_len, int *ret_len,
   return ASN1_SUCCESS;
 }
 
-static int
-_asn1_get_objectid_der (const unsigned char *der, int der_len, int *ret_len,
+/**
+ * asn1_get_objectid_der:
+ * @der: DER data to decode containing the OBJECT IDENTIFIER
+ * @der_len: Length of DER data to decode.
+ * @ret_len: Output variable containing the length of the DER data.
+ * @str: Pre-allocated output buffer to put the textual object id in.
+ * @str_size: Length of pre-allocated output buffer.
+ *
+ * Converts a DER encoded object identifier to its textual form.
+ *
+ * Returns: %ASN1_SUCCESS on success, or an error.
+ **/
+int
+asn1_get_object_id_der (const unsigned char *der, int der_len, int *ret_len,
 			char *str, int str_size)
 {
   int len_len, len, k;
@@ -370,7 +445,7 @@ _asn1_get_objectid_der (const unsigned char *der, int der_len, int *ret_len,
  *
  * Extract a BIT SEQUENCE from DER data.
  *
- * Returns: Return %ASN1_SUCCESS on success, or an error.
+ * Returns: %ASN1_SUCCESS on success, or an error.
  **/
 int
 asn1_get_bit_der (const unsigned char *der, int der_len,
@@ -683,6 +758,7 @@ _asn1_extract_der_octet (asn1_node node, const unsigned char *der,
     return ASN1_DER_ERROR;
 
   counter = len3 + 1;
+  DECR_LEN(der_len, len3);
 
   if (len2 == -1)
     counter_end = der_len - 2;
@@ -691,6 +767,7 @@ _asn1_extract_der_octet (asn1_node node, const unsigned char *der,
 
   while (counter < counter_end)
     {
+      DECR_LEN(der_len, 1);
       len2 = asn1_get_length_der (der + counter, der_len, &len3);
 
       if (IS_ERR(len2, flags))
@@ -715,7 +792,6 @@ _asn1_extract_der_octet (asn1_node node, const unsigned char *der,
 	  len2 = 0;
 	}
 
-      DECR_LEN(der_len, 1);
       counter += len2 + len3 + 1;
     }
 
@@ -726,15 +802,17 @@ cleanup:
 }
 
 static int
-_asn1_get_octet_string (asn1_node node, const unsigned char *der, int der_len,
+get_octet_string (asn1_node node, const unsigned char *der, int der_len,
+                        const unsigned char *tag, unsigned tag_len,
                         int *len, unsigned flags)
 {
   int len2, len3, counter, tot_len, indefinite;
   int result;
+  int orig_der_len = der_len;
 
   counter = 0;
 
-  if (*(der - 1) & ASN1_CLASS_STRUCTURED)
+  if (tag[0] & ASN1_CLASS_STRUCTURED)
     {
       tot_len = 0;
 
@@ -767,13 +845,19 @@ _asn1_get_octet_string (asn1_node node, const unsigned char *der, int der_len,
 
           DECR_LEN(der_len, 1);
 	  if (der[counter] != ASN1_TAG_OCTET_STRING)
-	    return ASN1_DER_ERROR;
+	    {
+	      warn();
+	      return ASN1_DER_ERROR;
+	    }
 
 	  counter++;
 
 	  len2 = asn1_get_length_der (der + counter, der_len, &len3);
 	  if (len2 <= 0)
-	    return ASN1_DER_ERROR;
+	    {
+	      warn();
+	      return ASN1_DER_ERROR;
+	    }
 
           DECR_LEN(der_len, len3 + len2);
 	  counter += len3 + len2;
@@ -792,9 +876,12 @@ _asn1_get_octet_string (asn1_node node, const unsigned char *der, int der_len,
 	  asn1_length_der (tot_len, temp, &len2);
 	  _asn1_set_value (node, temp, len2);
 
-	  ret = _asn1_extract_der_octet (node, der, der_len, flags);
+	  ret = _asn1_extract_der_octet (node, der, orig_der_len, flags);
 	  if (ret != ASN1_SUCCESS)
-	    return ret;
+	    {
+	      warn();
+	      return ret;
+	    }
 
 	}
     }
@@ -802,7 +889,10 @@ _asn1_get_octet_string (asn1_node node, const unsigned char *der, int der_len,
     {				/* NOT STRUCTURED */
       len2 = asn1_get_length_der (der, der_len, &len3);
       if (len2 < 0)
-	return ASN1_DER_ERROR;
+        {
+          warn();
+	  return ASN1_DER_ERROR;
+	}
 
       DECR_LEN(der_len, len3+len2);
       counter = len3 + len2;
@@ -1161,7 +1251,7 @@ asn1_der_decoding2 (asn1_node *element, const void *ider, int *max_ider_len,
 	      break;
 	    case ASN1_ETYPE_OBJECT_ID:
 	      result =
-		_asn1_get_objectid_der (der + counter, ider_len, &len2,
+		asn1_get_object_id_der (der + counter, ider_len, &len2,
 					temp, sizeof (temp));
 	      if (result != ASN1_SUCCESS)
 	        {
@@ -1181,8 +1271,8 @@ asn1_der_decoding2 (asn1_node *element, const void *ider, int *max_ider_len,
 	    case ASN1_ETYPE_GENERALIZED_TIME:
 	    case ASN1_ETYPE_UTC_TIME:
 	      result =
-		_asn1_get_time_der (der + counter, ider_len, &len2, temp,
-				    sizeof (temp) - 1);
+		_asn1_get_time_der (type_field (p->type), der + counter, ider_len, &len2, temp,
+				    sizeof (temp) - 1, flags);
 	      if (result != ASN1_SUCCESS)
 	        {
                   warn();
@@ -1199,7 +1289,15 @@ asn1_der_decoding2 (asn1_node *element, const void *ider, int *max_ider_len,
 	      move = RIGHT;
 	      break;
 	    case ASN1_ETYPE_OCTET_STRING:
-	      result = _asn1_get_octet_string (p, der + counter, ider_len, &len3, flags);
+	      if (counter < tag_len)
+	        {
+		  result = ASN1_DER_ERROR;
+                  warn();
+		  goto cleanup;
+	        }
+	      result = get_octet_string (p, der + counter, ider_len, 
+	      				 der + counter - tag_len, tag_len,
+	      				 &len3, flags);
 	      if (result != ASN1_SUCCESS)
 	        {
                   warn();
@@ -2023,7 +2121,7 @@ asn1_expand_octet_string (asn1_node definitions, asn1_node * element,
  * asn1_decode_simple_der:
  * @etype: The type of the string to be encoded (ASN1_ETYPE_)
  * @der: the encoded string
- * @der_len: the bytes of the encoded string
+ * @_der_len: the bytes of the encoded string
  * @str: a pointer to the data
  * @str_len: the length of the data
  *
@@ -2034,11 +2132,12 @@ asn1_expand_octet_string (asn1_node definitions, asn1_node * element,
  **/
 int
 asn1_decode_simple_der (unsigned int etype, const unsigned char *der,
-			unsigned int der_len, const unsigned char **str,
+			unsigned int _der_len, const unsigned char **str,
 			unsigned int *str_len)
 {
   int tag_len, len_len;
   const unsigned char *p;
+  int der_len = _der_len;
   unsigned char class;
   unsigned long tag;
   long ret;
@@ -2046,7 +2145,7 @@ asn1_decode_simple_der (unsigned int etype, const unsigned char *der,
   if (der == NULL || der_len == 0)
     return ASN1_VALUE_NOT_VALID;
 
-  if (ETYPE_OK (etype) == 0)
+  if (ETYPE_OK (etype) == 0 || ETYPE_IS_STRING(etype) == 0)
     return ASN1_VALUE_NOT_VALID;
 
   /* doesn't handle constructed classes */
@@ -2063,6 +2162,8 @@ asn1_decode_simple_der (unsigned int etype, const unsigned char *der,
 
   p += tag_len;
   der_len -= tag_len;
+  if (der_len <= 0)
+    return ASN1_DER_ERROR;
 
   ret = asn1_get_length_der (p, der_len, &len_len);
   if (ret < 0)
@@ -2070,9 +2171,182 @@ asn1_decode_simple_der (unsigned int etype, const unsigned char *der,
 
   p += len_len;
   der_len -= len_len;
+  if (der_len <= 0)
+    return ASN1_DER_ERROR;
 
   *str_len = ret;
   *str = p;
 
   return ASN1_SUCCESS;
 }
+
+static int append(uint8_t **dst, unsigned *dst_size, const unsigned char *src, unsigned src_size)
+{
+  *dst = realloc(*dst, *dst_size+src_size);
+  if (*dst == NULL)
+    return ASN1_MEM_ERROR;
+  memcpy(*dst + *dst_size, src, src_size);
+  *dst_size += src_size;
+  return ASN1_SUCCESS;
+}
+
+/**
+ * asn1_decode_simple_ber:
+ * @etype: The type of the string to be encoded (ASN1_ETYPE_)
+ * @der: the encoded string
+ * @_der_len: the bytes of the encoded string
+ * @str: a pointer to the data
+ * @str_len: the length of the data
+ * @ber_len: the total length occupied by BER (may be %NULL)
+ *
+ * Decodes a BER encoded type. The output is an allocated value 
+ * of the data. This decodes BER STRINGS only. Other types are
+ * decoded as DER.
+ *
+ * Returns: %ASN1_SUCCESS if successful or an error value.
+ **/
+int
+asn1_decode_simple_ber (unsigned int etype, const unsigned char *der,
+			unsigned int _der_len, unsigned char **str,
+			unsigned int *str_len, unsigned int *ber_len)
+{
+  int tag_len, len_len;
+  const unsigned char *p;
+  int der_len = _der_len;
+  uint8_t *total = NULL;
+  unsigned total_size = 0;
+  unsigned char class;
+  unsigned long tag;
+  unsigned char *out = NULL;
+  unsigned out_len;
+  long ret;
+
+  if (ber_len) *ber_len = 0;
+
+  if (der == NULL || der_len == 0)
+    {
+      warn();
+      return ASN1_VALUE_NOT_VALID;
+    }
+
+  if (ETYPE_OK (etype) == 0)
+    {
+      warn();
+      return ASN1_VALUE_NOT_VALID;
+    }
+
+  /* doesn't handle constructed classes */
+  if (ETYPE_CLASS (etype) != ASN1_CLASS_UNIVERSAL)
+    {
+      warn();
+      return ASN1_VALUE_NOT_VALID;
+    }
+
+  p = der;
+  ret = asn1_get_tag_der (p, der_len, &class, &tag_len, &tag);
+  if (ret != ASN1_SUCCESS)
+    {
+      warn();
+      return ret;
+    }
+
+  if (ber_len) *ber_len += tag_len;
+
+  if (tag != ETYPE_TAG (etype))
+    {
+      warn();
+      return ASN1_DER_ERROR;
+    }
+
+  p += tag_len;
+  der_len -= tag_len;
+  if (der_len <= 0)
+    return ASN1_DER_ERROR;
+
+  if (class == ASN1_CLASS_STRUCTURED && ETYPE_IS_STRING(etype))
+    {
+
+      len_len = 1;
+      if (p[0] != 0x80)
+        {
+          warn();
+          return ASN1_DER_ERROR;
+        }
+
+      p += len_len;
+      der_len -= len_len;
+      if (der_len <= 0)
+        return ASN1_DER_ERROR;
+
+      if (ber_len) *ber_len += len_len;
+
+      /* decode the available octet strings */
+      do
+        {
+          unsigned tmp_len;
+
+          ret = asn1_decode_simple_ber(etype, p, der_len, &out, &out_len, &tmp_len);
+          if (ret != ASN1_SUCCESS)
+            {
+              free(total);
+              return ret;
+            }
+          p += tmp_len;
+          der_len -= tmp_len;
+          if (ber_len) *ber_len += tmp_len;
+
+          if (der_len < 2) /* we need the EOC */
+            {
+              free(total);
+              return ASN1_DER_ERROR;
+            }
+
+	  if (out_len > 0)
+	    {
+              ret = append(&total, &total_size, out, out_len);
+              free(out);
+              if (ret != ASN1_SUCCESS)
+                {
+                  free(total);
+                  return ret;
+                }
+	    }
+
+	  if (p[0] == 0 && p[1] == 0) /* EOC */
+	    {
+              if (ber_len) *ber_len += 2;
+  	      break;
+  	    }
+        }
+      while(1);
+    }
+  else if (class == ETYPE_CLASS(etype))
+    {
+      if (ber_len)
+        {
+          ret = asn1_get_length_der (p, der_len, &len_len);
+          if (ret < 0)
+            {
+              warn();
+              return ASN1_DER_ERROR;
+            }
+          *ber_len += ret + len_len;
+        }
+
+      /* non-string values are decoded as DER */
+      ret = asn1_decode_simple_der(etype, der, _der_len, (const unsigned char**)&out, &out_len);
+      if (ret != ASN1_SUCCESS)
+        return ret;
+
+      ret = append(&total, &total_size, out, out_len);
+      if (ret != ASN1_SUCCESS)
+        return ret;
+    }
+  else
+    return ASN1_DER_ERROR;
+
+  *str = total;
+  *str_len = total_size;
+
+  return ASN1_SUCCESS;
+}
