@@ -44,7 +44,7 @@ extern "C"
 {
 #endif
 
-#define ASN1_VERSION "4.1"
+#define ASN1_VERSION "4.6"
 
 #if defined(__GNUC__) && !defined(ASN1_INTERNAL_BUILD)
 # define _ASN1_GCC_VERSION (__GNUC__ * 10000 + __GNUC_MINOR__ * 100 + __GNUC_PATCHLEVEL__)
@@ -330,6 +330,13 @@ extern "C"
 				const unsigned char **str,
 				unsigned int *str_len);
 
+  extern ASN1_API
+    int asn1_decode_simple_ber (unsigned int etype, const unsigned char *der,
+				unsigned int der_len,
+				unsigned char **str,
+				unsigned int *str_len,
+				unsigned int *ber_len);
+
   extern ASN1_API int
     asn1_encode_simple_der (unsigned int etype, const unsigned char *str,
 			    unsigned int str_len, unsigned char *tl,
@@ -367,6 +374,11 @@ extern "C"
 		      int *ret_len, unsigned char *str,
 		      int str_size, int *bit_len);
 
+  extern ASN1_API int
+    asn1_get_object_id_der (const unsigned char *der,
+                            int der_len, int *ret_len,
+		 	    char *str, int str_size);
+
 /* Compatibility types */
 
   typedef int asn1_retCode;	/* type returned by libtasn1 functions */
