@@ -109,6 +109,30 @@ main (int argc, char *argv[])
       exit (1);
     }
 
+  result = asn1_write_value (asn1_element, "a", "string1", 7);
+  if (result != ASN1_SUCCESS)
+    {
+      fprintf (stderr, "asn1_write_value(): str ");
+      asn1_perror (result);
+      exit (1);
+    }
+
+  result = asn1_write_value (asn1_element, "b", "string2", 7);
+  if (result != ASN1_SUCCESS)
+    {
+      fprintf (stderr, "asn1_write_value(): str ");
+      asn1_perror (result);
+      exit (1);
+    }
+
+  result = asn1_write_value (asn1_element, "c", "string3", 7);
+  if (result != ASN1_SUCCESS)
+    {
+      fprintf (stderr, "asn1_write_value(): str ");
+      asn1_perror (result);
+      exit (1);
+    }
+
   /* Clear the definition structures */
   asn1_delete_structure (&definitions);
 
